package com.mcwl.web.controller.memberCenter;

import com.mcwl.common.core.domain.AjaxResult;
import com.mcwl.common.core.domain.entity.SysUser;
import com.mcwl.common.utils.SecurityUtils;
import com.mcwl.memberCenter.domain.MemberConsume;
import com.mcwl.memberCenter.domain.Member;
import com.mcwl.memberCenter.domain.MemberLevel;
import com.mcwl.memberCenter.service.MemberBenefitService;
import com.mcwl.memberCenter.domain.dto.RechargePointsDto;
import com.mcwl.memberCenter.domain.dto.UserMemberDto;
import com.mcwl.memberCenter.service.MemberConsumeService;
import com.mcwl.memberCenter.service.MemberLevelService;
import com.mcwl.memberCenter.service.MemberService;
import com.mcwl.system.service.ISysUserService;
import com.mcwl.memberCenter.domain.vo.PointsVO;
import lombok.RequiredArgsConstructor;
import org.apache.commons.lang3.StringUtils;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import javax.validation.constraints.NotNull;
import java.util.List;
import java.util.Optional;

@RestController
@RequestMapping("member")
@RequiredArgsConstructor
public class MemberController {

    private final MemberService memberService;

    private final ISysUserService sysUserService;

    private final MemberConsumeService memberConsumeService;

    private final MemberBenefitService memberBenefitService;

    private final MemberLevelService memberLevelService;

    /**
     * 创建用户会员
     *
     * @param userMemberDto 参数
     * @return 用户会员
     */
    @PostMapping("createMember")
    public AjaxResult createMemberCenter(@RequestBody @Valid UserMemberDto userMemberDto) {
        Long userId = userMemberDto.getUserId();
        Long memberLevelId = userMemberDto.getMemberLevelId();
        String paymentMethod = userMemberDto.getPaymentMethod();
        Long promotionId = userMemberDto.getPromotionId();

        SysUser sysUser = sysUserService.selectUserById(userId);
        if (!Optional.ofNullable(sysUser).isPresent()) {
            return AjaxResult.warn("用户不存在");
        }

        MemberLevel memberLevel = memberLevelService.getById(memberLevelId);
        if (!Optional.ofNullable(memberLevel).isPresent()) {
            return AjaxResult.warn("会员等级不存在");
        }

        Member member = memberService.createUserMember(userId, memberLevelId, paymentMethod, promotionId);
        if (!Optional.ofNullable(member).isPresent()) {
            return AjaxResult.warn("创建会员失败");
        }

        return AjaxResult.success(member);
    }


    /**
     * 获取积分余额和历史记录
     *
     * @return 积分余额和历史记录
     */
    @GetMapping("getPoints")
    public AjaxResult getPoints() {
        Long userId = SecurityUtils.getUserId();

        Member member = memberService.getUseUserMemberByUserId(userId);
        if (!Optional.ofNullable(member).isPresent()) {
            return AjaxResult.warn("用户未开通会员");
        }

        PointsVO pointsVO = new PointsVO();

        // 会员消费记录
        List<MemberConsume> memberConsumeList = memberConsumeService
                .lambdaQuery()
                .eq(MemberConsume::getUserId, userId)
                .list();

        pointsVO.setPoints(member.getPoints());
        pointsVO.setMemberConsumeList(memberConsumeList);

        return AjaxResult.success(pointsVO);
    }


    /**
     * 会员积分充值
     */
    @PostMapping("rechargePoints")
    public AjaxResult rechargePoints(@RequestBody @Valid RechargePointsDto rechargePointsDto) {
        Long userId = rechargePointsDto.getUserId();
        Double amount = rechargePointsDto.getAmount();

        SysUser sysUser = sysUserService.selectUserById(userId);
        if (!Optional.ofNullable(sysUser).isPresent()) {
            return AjaxResult.warn("用户不存在");
        }

        Member member = memberService.rechargePoints(userId, amount * 100);

        // 返回充值积分
        if (!Optional.ofNullable(member).isPresent()) {
            return AjaxResult.warn("充值积分失败");
        }

        rechargePointsDto.setPoints(member.getPoints());

        return AjaxResult.success(rechargePointsDto);
    }


    /**
     * 消费积分
     */
    @GetMapping("consumePoints/{consumePoints}")
    public AjaxResult consumePoints(@PathVariable Double consumePoints) {

        if (consumePoints == null || consumePoints <= 0) {
            return AjaxResult.warn("消费积分不能为空或小于0");
        }

        memberService.consumePoints(consumePoints);
        return AjaxResult.success();

    }


    /**
     * 根据会员等级和活动计算支付金额
     */
    @GetMapping("calculatePayment")
    public AjaxResult calculatePayment(@NotNull(message = "请选择会员") Long memberLevelId, @RequestParam(required = false) Long promotionId) {
        MemberLevel memberLevel = memberLevelService.getById(memberLevelId);
        Double unitPrice = memberLevel.getUnitPrice();
        unitPrice = memberService.calculatePayment(unitPrice, promotionId);
        return AjaxResult.success(unitPrice);
    }


}
